#!/usr/bin/env python

"""
 smtp2zope.py

 Reads a email from stdin and forwards it to ZOPE
 
 Usage: smtp2zope.py [-r] callURL [maxBytes]
 
 -r       = optional:
            submit only r[aw] mail-text without attachments to ZOPE

 callURL  = call this URL with the email as a post-request

 maxBytes = optional, but highly recommended:             
	    only forward requests < maxBytes to ZOPE
	    (maxBytes is the maximum size of the email after urlencoding)
 
 For automatic processing emails via your MTA,
 add a new alias to '/etc/aliases', then call 'newaliases'.

 yourAlias: "|smtp2http.py [-r] callUrl [maxBytes]"

 Please send bug-reports to: maik.jablonski@uni-bielefeld.de 

 #####
 ###  smtp2zope.py - a smtp-to-zope-gateway for ZOPE
 ##   Copyright (C) 2002   Maik Jablonski (maik.jablonski@uni-bielefeld.de)
 #
 #    This program is free software; you can redistribute it and/or modify
 #    it under the terms of the GNU General Public License as published by
 #    the Free Software Foundation; either version 2 of the License, or
 #    (at your option) any later version.
 #
 #    This program is distributed in the hope that it will be useful,
 #    but WITHOUT ANY WARRANTY; without even the implied warranty of
 #    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 #    GNU General Public License for more details.
 #
 #    You should have received a copy of the GNU General Public License
 ##   along with this program; if not, write to the Free Software
 ###  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #####
"""

import sys, StringIO, multifile, mimetools, urllib, mimify, getopt, re

# By default all Attachments will be submitted to ZOPE

sendAttachments=1

# Initialise a Dictionary for storing the mail-data

zopeMail = {'headers':{},'attachments':[],'body':''}


def unpack_multifile(multifile):
   """ Given a multifile, all attachments will be mime-decoded 
       and stored in the zopeMail-Dictionary """

   global zopeMail
   
   msg      = mimetools.Message(multifile)
   maintype = msg.getmaintype()
   subtype  = msg.getsubtype()
   name     = msg.getparam('name')

   # Recurse over all nested multiparts

   if maintype == 'multipart':
       multifile.push(msg.getparam('boundary'))
       multifile.readlines()
       while not multifile.last:
           multifile.next()
           unpack_multifile(multifile)
       multifile.pop()
       return

   # Process MIME-encoded data

   plainfile = StringIO.StringIO()

   try:
       mimetools.decode(multifile,plainfile,msg.getencoding())

   # An unknown or no encoding? 7bit or 8bit or whatever... copy literal

   except ValueError:
       mimetools.copyliteral(multifile,plainfile)

   # Get the plain text as the body of the mail...
   
   if maintype=='text' and subtype=='plain' and not name:
       zopeMail['body'] = plainfile.getvalue()
       
   # Decode all the rest
  
   else:

       # No name? This is highly propably the html-encoded mail-body...

       if not name:
           name = '%s.%s' % (maintype,subtype)

       # Store the attachment in Dictionary as a list of (name,content,content-type)
		
       zopeMail['attachments'].append([name,plainfile.getvalue(),maintype+'/'+subtype])

   plainfile.close()


## Mainpart of the script

# Check if we have at least one parameter 

if len(sys.argv) == 1:
	print __doc__
	sys.exit(1)

# Check for optional r[aw]-option

(optlist,args)=getopt.getopt(sys.argv[1:],"r")

if ('-r','') in optlist:
	sendAttachments=0

# Get the url to call

callURL   = args[0]

# Check for optional maxBytes

if len(args) == 2:
	maxBytes  = int(args[1])
else:
	maxBytes  = 0 # means: unlimited!!!

# Decode the email

rawData = sys.stdin.read()
msg     = mimetools.Message(StringIO.StringIO(rawData))  

# Extract the required email-headers & unmimify them

for (key,value) in msg.items():
	zopeMail['headers'][key] = mimify.mime_decode_header(value)
   
# Process mime encoded mail messages... works for plain mails as well...;-)

unpack_multifile(multifile.MultiFile(StringIO.StringIO(rawData)))

# Urlencode attachments as ZOPE-records

attachments=''

for (filename,filecontent,filetype) in zopeMail['attachments']:
	attachments = attachments + '&attachments.filename:records='    + urllib.quote(re.sub('[^\w.-]','',filename))
	attachments = attachments + '&attachments.filecontent:records=' + urllib.quote(filecontent)
	attachments = attachments + '&attachments.filetype:records='    + urllib.quote(filetype)

# Urlencode the headers, body & attachments for http-request

postData=urllib.urlencode(zopeMail['headers'])+'&body='+urllib.quote(zopeMail['body'])

# If -r(aw)-option is given, no attachments will be transfered to Zope!

if sendAttachments:
	postData = postData + attachments
	
# If the postData is not too big, call URL with data as a post-request

if not maxBytes or len(postData)<maxBytes:
	urllib.urlopen(callURL,data=postData)
